use crate::{db, server};

pub fn serve(config: &crate::config::Config) -> u8 {
    tracing_subscriber::fmt::init();

    tracing::info!("Initializing database and running migrations");

    let pool = db::initialize_db(config.data_dir.join(crate::DB_FILENAME), true).unwrap();
    let repo = db::SqliteRepository::from(pool);

    let ctx = server::Context {
        store: crate::gpodder::GpodderRepository::new(repo),
    };
    let app = server::app(ctx);

    let rt = tokio::runtime::Builder::new_multi_thread()
        .enable_all()
        .build()
        .unwrap();

    let address = format!("{}:{}", config.domain, config.port);
    tracing::info!("Starting server on {address}");

    rt.block_on(async {
        let listener = tokio::net::TcpListener::bind(address).await.unwrap();
        axum::serve(listener, app.into_make_service())
            .await
            .unwrap()
    });

    0
}
